(function($){

	var defaultFontColor = '#000',
		defaultBackgroundColor = '#fff',
		fontColorCommand = 'fontColor',
		backgroundColorCommand = 'backgroundColor';

	function colorCommand(name, style){
		this.name = name;
		this.style = style;
	}

	colorCommand.prototype = {
		exec : function( editor ){

			var self = this;

			// use ep.ui.colorpicker for color selection
			$.ready('ep.ui.colorpicker', function(){
				var button = ep('#'+self.uiItems[0]._.id),
				userColors = {};

				for(var key in epConfig.userDefinedColors){
					userColors['Color'+(Number(key)+1)] = epConfig.userDefinedColors[key];
				}

				button
					.uiColorpicker({
						autoOpen:       true,
						color:          self.name === backgroundColorCommand ? defaultBackgroundColor : defaultFontColor,
						orientation:    'bottom',
						transparent:    self.name === backgroundColorCommand ? true : false,
						callback:       function( color ){
							editor.fire( 'saveSnapshot' );
							new CKEDITOR.style( self.style, { color : color } ).apply( editor.document );
							editor.fire( 'saveSnapshot' );
						},
						extended: {
							userColors: userColors
						}
					});
			});
		}
	};

	CKEDITOR.plugins.add( 'epcolorbutton',
	{
		requires : [ 'styles', 'domiterator' ],
		init : function( editor ){
			var config = editor.config,
				lang = editor.lang.colorButton;

			// add commands
			editor.addCommand( fontColorCommand, new colorCommand( fontColorCommand, config.colorButton_foreStyle ) );
			editor.addCommand( backgroundColorCommand, new colorCommand( backgroundColorCommand, config.colorButton_backStyle ) );

			// add toolbar buttons
			addButton( 'eptextcolor', lang.textColorTitle, fontColorCommand, this.path+'images/fontcolor.png' );
			addButton( 'epbgcolor', lang.bgColorTitle, backgroundColorCommand, this.path+'images/backgroundcolor.png' );

			function addButton( name, title, commandName, icon ){
				editor.ui.addButton( name,{
					label: title,
					command: commandName,
					icon: icon
				});
			}

			// handle button selection
			editor.on( 'selectionChange', function(event){
				var element = editor.getSelection().getStartElement(),
					hasFontColor = false,
					hasBackgroundColor = false;

				// return if no element was selected
				if ( !element ){
					return;
				}

				// iterate over the selected element and its parents
				while(element.getName() !== 'body'){
					var backgroundColor = element.getStyle('background-color'),
						fontColor = element.getStyle('color');

					// check if the current element has a font or background color
					if(fontColor){
						hasFontColor = true;
						defaultFontColor = fontColor;
					}

					if(backgroundColor){
						hasBackgroundColor = true;
						defaultBackgroundColor = backgroundColor;
					}

					// get next element(parent)
					element = element.getParent();
				}

				// set state of toolbar buttons
				editor.getCommand(fontColorCommand).setState(hasFontColor ? CKEDITOR.TRISTATE_ON : CKEDITOR.TRISTATE_OFF);
				editor.getCommand(backgroundColorCommand).setState(hasBackgroundColor ? CKEDITOR.TRISTATE_ON : CKEDITOR.TRISTATE_OFF);
			});
		}
	})
}(jQuery))